"""Creating word to word graph"""
from typing import Any

from hashlib import sha256

import pandas as pd

from phrase_counter.cleaner import cleaner


def generate_word_graph(doc: str) -> Any:
    """Main function for creating word to word graph"""
    cleaned_text = cleaner(doc)  # Cleaning

    words_list = []
    hashed_list = []
    from_list = []
    to_list = []

    # -------------------- Generating Graphs --------------------
    for text_part in cleaned_text.split("."):
        # Generating words
        words = text_part.strip().split(" ")

        words = [word for word in words if word != ""]  # Removing empty string

        # Hashing words
        hashed_words = [
            sha256(word.encode()).hexdigest() for word in words
        ]
        # Creating words relation
        from_rel = hashed_words[: len(hashed_words) - 1]
        to_rel = hashed_words[1:]

        # Extending results
        words_list.extend(words)
        hashed_list.extend(hashed_words)
        from_list.extend(from_rel)
        to_list.extend(to_rel)

    # -------------------- Concating Results --------------------
    words_df = pd.DataFrame(zip(words_list, hashed_list), columns=["word", "word_hash"])
    words_df["count"] = 1

    rel_df = pd.DataFrame(zip(from_list, to_list), columns=["_from", "_to"])
    rel_df["count"] = 1

    # -------------------- Aggregating --------------------
    words_df = (
        words_df.groupby(["word", "word_hash"]).agg({"count": "sum"}).reset_index()
    )

    rel_df = rel_df.groupby(["_from", "_to"]).agg({"count": "sum"}).reset_index()

    return words_df, rel_df



"""Cleaning dirty input text."""
from typing import Any, List, Optional

import re
from html.parser import HTMLParser
from io import StringIO

import requests
from bs4 import BeautifulSoup
from cleaning_utils import clear_stop_char, replace_arabic_char
from polyglot.detect import Detector
from polyglot.detect.base import UnknownLanguage


def fetch_page_text(url: str = "", webpage: str = "") -> str:
    """Getting html pages from urls & fetching needed elements of the page.

    Args:
        url: url of the target webpage.
        webpage: string version of the webpage.

    Returns:
        text of the webpage.
    """
    # If url is defined then make the request and fetch the page.
    if url != "":
        req = requests.get(url=url)
        soup = BeautifulSoup(req.content, "html.parser")

    # If not then make soup from given webpage.
    else:
        soup = BeautifulSoup(webpage, "html.parser")

    # Getting needed tags and stripped strings
    whole_page = ""
    for group in soup(["h1", "h2", "h3", "h4", "h5", "h6", "p"]):
        for text in group.stripped_strings:
            whole_page += text + " "

    whole_page = whole_page.strip()

    return whole_page


def cleaner(
    dirty_text: str,
    remove_stop_regex: Optional[List[Any]] = None,
    remove_highlight_regex: Optional[List[Any]] = None
    # stop_list: Optional[Iterable[str]] = None,
) -> str:
    """Main function for cleaning.

    Args:
        dirty_text: Input dirty text
        remove_stop_regex: Regex for removing stop phrases
        remove_highlight_regex: Regex for removing highlight phrases

    Returns:
        Final text ready for integration in NLP algorithms.
    """
    # ------------------- HTML Stripper -------------------
    processed_text = BeautifulSoup(dirty_text, features="html.parser").get_text()
    # ------------------- Langugae detection -------------------
    try:
        detector = Detector(processed_text)
        lang = detector.language.code
    except UnknownLanguage:
        lang = ""
    # ------------------- Linguistic phase -------------------
    if lang == "fa":
        processed_text = replace_arabic_char(processed_text)
    elif lang == "ar":
        processed_text = replace_arabic_char(processed_text, letter=False)

    processed_text = clear_stop_char(
        processed_text,
        replace_char=".",
    )

    # ------------------- Trimmer phase -------------------
    trim_pattern = re.compile(
        "\t|\n|\u200c|\u200f|\u200e|\xa0|\xad|\r"
    )
    processed_text = re.sub(trim_pattern, " ", processed_text)
    processed_text = re.sub(" +", " ", processed_text)  # space cleaner
    processed_text = processed_text.strip()

    # --------------------- Remove frequents ---------------------
    if remove_stop_regex:
        for i in remove_stop_regex:
            processed_text = re.sub(remove_stop_regex[i], "", processed_text)

    if remove_highlight_regex:
        for i in remove_highlight_regex:
            processed_text = re.sub(remove_highlight_regex[i], "", processed_text)

    processed_text = re.sub(" +", " ", processed_text)  # space cleaner
    processed_text = processed_text.strip()

    return str(processed_text)



"""Main functions for counting phrases."""
from typing import Any, List, Optional, Tuple

from hashlib import sha256

import pandas as pd
from sklearn.feature_extraction.text import CountVectorizer

from phrase_counter.cleaner import cleaner, fetch_page_text


def ingest_doc(
    doc: str,
    doc_type: str = "TEXT",
    ngram_range: Tuple[int, int] = (1, 5),
    remove_stop_regex: Optional[List[Any]] = None,
    remove_highlight_regex: Optional[List[Any]] = None,
) -> Any:
    """Counting phrases in the document.

    Args:
        doc: Document to be processed.
        doc_type: URL, TEXT, or HTML.
        ngram_range: Determining bag of words length.
        remove_stop_regex: Regex for removing stop phrases.
        remove_highlight_regex: Regex for removing highlight phrases.

    Returns:
        Dataframe with counts for each word.

    Raises:
        Exception: If unkown value is given for doc_type.
    """
    # ---------------- Fetching the page text ----------------
    # Fetching a page from URL
    if doc_type == "URL":
        dirty_text = fetch_page_text(url=doc)

    # Processing HTML
    elif doc_type == "HTML":
        dirty_text = fetch_page_text(webpage=doc)

    # Raw text
    elif doc_type == "TEXT":
        dirty_text = doc

    else:
        raise Exception("Unknown value for doc_type argument.")

    cleaned_text = cleaner(
        dirty_text,
        remove_stop_regex=remove_stop_regex,
        remove_highlight_regex=remove_highlight_regex,
    )

    bags_list = []
    counts_list = []
    # ----------------- Splitting -----------------
    for text_part in cleaned_text.split("."):
        # ----------------- Counter Section -----------------
        # Initializing vector
        count_vector = CountVectorizer(
            ngram_range=(ngram_range[0], ngram_range[1]), encoding="utf-8"
        )
        try:
            # Fit on text
            count_data = count_vector.fit_transform([text_part.strip()])
        except ValueError:
            continue
        # ----------------- Extending data -----------------
        bags_list.extend(count_vector.get_feature_names_out().tolist())
        counts_list.extend(count_data.toarray()[0].tolist())

    # Concating results
    phrase_df = pd.DataFrame(zip(bags_list, counts_list), columns=["bag", "count"])

    # Aggregating for removing duplicate values (rows with same hash or phrase)
    phrase_df = phrase_df.groupby(["bag"]).agg({"count": "sum"}).reset_index()

    # Creating phrase hash
    phrase_df["_key"] = [sha256(bag.encode()).hexdigest() for bag in phrase_df["bag"]]

    # Counting number of words in each bag
    phrase_df["length"] = [len(str(bag).split()) for bag in phrase_df["bag"]]

    return phrase_df
